"""This module is used to Build the Employer object and associated items for the EAM ONB Helper."""

import uuid
import time

from .common import helpers


class EmpBuilder:
    """
    This class is used to build the Employer object and associated items for the EAM ONB Helper.
    """

    def __init__(self, data_base, company_data: list) -> None:
        self.company_id = str(uuid.uuid4())
        self.company_name = company_data[0]
        self.data_base = data_base
        self.company_data = company_data

    @staticmethod
    def format(item: dict) -> dict:
        """This method is used to format the item."""
        item['id'] = str(uuid.uuid4())
        item['type'] = item['itemType']
        del item['itemType']

        return item

    def build_company(self) -> dict:
        """This method is used to build the company."""
        company = self.data_base.get_result(
            query="SELECT * FROM c WHERE c.type = 'template' AND c.itemType = 'company'"
        )
        company['id'] = self.company_id
        company['name'] = self.company_data[0]
        company['type'] = 'company'
        del company['itemType']
        company['accountType'] = 'employer'
        company['email'] = self.company_data[13]
        company['isDemo'] = self.company_data[1]
        company['signUpCode'] = helpers.generate_sign_up_code()
        company['website'] = self.company_data[2]
        company['address'] = {
            'street': self.company_data[3],
            'city': self.company_data[4],
            'province': self.company_data[5],
            'postalCode': self.company_data[6],
            'country': self.company_data[7],
            'office': self.company_data[8]
        }
        company['phone'] = self.company_data[9]
        company['dates']['added'] = int(time.time())

        return company

    def build_tags(self) -> dict:
        """This method is used to build the tags."""
        tags = self.format(
            self.data_base.get_result(
                query="SELECT * FROM c WHERE c.type = 'template' AND c.itemType = 'tags'"
            )
        )
        tags['companyId'] = self.company_id
        return tags

    def build_employer_doc_config(self) -> dict:
        """This method is used to build the employer document configuration."""
        emp_doc_config = self.format(
            self.data_base.get_result(
                query="SELECT * FROM c WHERE c.type = 'template'"
                "AND c.itemType = 'employerDocConfig'"
            )
        )
        emp_doc_config['employerId'] = self.company_id
        emp_doc_config['employerName'] = self.company_name
        return emp_doc_config

    def build_updated_employer_doc_config(self) -> dict:
        """This method is used to build the updated employer document configuration."""
        emp_doc_config = self.format(
            self.data_base.get_result(
                query="SELECT * FROM c WHERE c.type = 'template'"
                "AND c.itemType = 'docConfig'"
            )
        )
        emp_doc_config['employerId'] = self.company_id
        emp_doc_config['employerName'] = self.company_name
        return emp_doc_config

  # ADD MANUALLY FOR NOW
  # def build_questions(self):
  #   pass

  # ADD MANUALLY FOR NOW
  # def build_doc_notes(self):
  #   pass



"""This module is used to test the functions in the helpers module."""

from unittest import mock

# from eightam_onb_helper.common.helpers import generate_sign_up_code
from eightam_onb_helper.src.eam_onb_helper.common.helpers import generate_sign_up_code


@mock.patch(
    'eightam_onb_helper.src.eam_onb_helper.common.helpers.'\
        'geonamescache.GeonamesCache.get_us_counties',
    return_value=[{'name': 'test_county'}, {'name': 'test_county'}]
)
@mock.patch('random.randint', return_value=1234)
#pylint: disable=unused-argument
def test_generate_sign_up_code(mock_choice, mock_randint):
    """This method is used to test the generate_sign_up_code method."""
    assert generate_sign_up_code() == 'test_county1234'



"""This module is used to test the VendBuilder class."""

from unittest import mock, TestCase

from eightam_onb_helper.tests.common.cosmos_client_mock import CosmosClientMock
from eightam_onb_helper.src.eam_onb_helper.vend import VendBuilder
from eightam_db_helper.src.eam_db_helper.db import DatabaseHelper


DATA_BASE = DatabaseHelper(
    {
        'account_uri': 'test_uri',
        'key': 'test_key',
        'db_name': 'test_database',
        'container_name': 'test_container'
    },
    CosmosClientMock
)

#pylint: disable=unused-argument
class TestVendBuilder(TestCase):
    """This class is used to test the VendBuilder class."""
    @mock.patch('uuid.uuid4', return_value='test_id')
    def test_format(self, mock_uuid4):
        """This method is used to test the format method."""
        vend_builder = VendBuilder(DATA_BASE, ['test_company_name'])
        item = {
            'itemType': 'company'
        }
        formatted_item = vend_builder.format(item)
        self.assertEqual(formatted_item, {'id': 'test_id', 'type': 'company'})


    @mock.patch('uuid.uuid4', return_value='test_company_id')
    @mock.patch(
        'eightam_onb_helper.src.eam_onb_helper.common.helpers.generate_sign_up_code',
        return_value='test_sign_up_code'
    )
    @mock.patch('time.time', return_value=123456)
    def test_build_company(self, mock_uuid4, mock_signup_code, mock_time):
        """This method is used to test the build_company method."""
        DATA_BASE.client.results = [{'dates': {'added': 123456}, 'itemType': 'company'}]
        vend_builder = VendBuilder(
            DATA_BASE,
            [
                'test_company_name',
                'test_demo',
                'test_subscription_type',
                'test_website',
                'test_street',
                'test_city',
                'test_province',
                'test_postal_code',
                'test_country',
                'test_office',
                'test_phone',
                '',
                '',
                '',
                'test_email'
            ]
        )
        company = vend_builder.build_company()

        self.assertEqual(company, {
            'id': 'test_company_id',
            'name': 'test_company_name',
            'email': 'test_email',
            'isDemo': 'test_demo',
            'signUpCode': 'test_sign_up_code',
            'subscriptionType': 'test_subscription_type',
            'website': 'test_website',
            'address': {
                'street': 'test_street',
                'city': 'test_city',
                'province': 'test_province',
                'postalCode': 'test_postal_code',
                'country': 'test_country',
                'office': 'test_office'
            },
            'phone': 'test_phone',
            'dates': {'added': 123456},
            'type': 'company'
        })


    @mock.patch('uuid.uuid4', return_value='test_company_id')
    def test_build_profile_state(self, mock_uuid4):
        """Test the build_profile_state method."""
        DATA_BASE.client.results = [{
            "itemType": "vendorPreQualProfileState"
        }]

        vend_builder = VendBuilder(DATA_BASE, ['test_company_name'])
        profile_state = vend_builder.build_profile_state()
        self.assertEqual(profile_state, {
            "id": "test_company_id",
            "type": "vendorPreQualProfileState",
            "vendorId": "test_company_id",
            "vendorName": "test_company_name",
        })


    @mock.patch('uuid.uuid4', return_value='test_company_id')
    def build_payment_state(self, mock_uuid4):
        """Test the build_payment_state method."""
        DATA_BASE.client.results = [{
            "itemType": "vendorPreQualPaymentState"
        }]

        vend_builder = VendBuilder(DATA_BASE, ['test_company_name'])
        payment_state = vend_builder.build_payment_state()
        self.assertEqual(payment_state, {
            "id": "test_company_id",
            "type": "vendorPreQualPaymentState",
            "vendorId": "test_company_id",
            "vendorName": "test_company_name",
        })


    @mock.patch('uuid.uuid4', return_value='test_company_id')
    def test_build_documents_state(self, mock_uuid4):
        """Test the build_documents_state method."""
        DATA_BASE.client.results = [{
            "itemType": "vendorPreQualDocumentsState"
        }]

        vend_builder = VendBuilder(DATA_BASE, ['test_company_name'])
        documents_state = vend_builder.build_documents_state()
        self.assertEqual(documents_state, {
            "id": "test_company_id",
            "type": "vendorPreQualDocumentsState",
            "vendorId": "test_company_id",
            "vendorName": "test_company_name",
        })


    @mock.patch('uuid.uuid4', return_value='test_company_id')
    def test_build_requirements_state(self, mock_uuid4):
        """Test the build_requirements_state method."""
        DATA_BASE.client.results = [{
            "itemType": "vendorPreQualRequirementsState"
        }]

        vend_builder = VendBuilder(DATA_BASE, ['test_company_name'])
        requirements_state = vend_builder.build_requirements_state()
        self.assertEqual(requirements_state, {
            "id": "test_company_id",
            "type": "vendorPreQualRequirementsState",
            "vendorId": "test_company_id",
            "vendorName": "test_company_name",
        })


    @mock.patch('uuid.uuid4', return_value='test_company_id')
    def test_build_stepper_state(self, mock_uuid4):
        """Test the build_stepper_state method."""
        DATA_BASE.client.results = [{
            "itemType": "vendorStepperState"
        }]

        vend_builder = VendBuilder(DATA_BASE, ['test_company_name'])
        stepper_state = vend_builder.build_stepper_state()
        self.assertEqual(stepper_state, {
            "id": "test_company_id",
            "type": "vendorStepperState",
            "vendorId": "test_company_id",
            "vendorName": "test_company_name",
        })


    @mock.patch('uuid.uuid4', return_value='test_id')
    def test_build_additional_req(self, mock_uuid4):
        """Test the build_additional_requirements method."""
        DATA_BASE.client.results = [{'text': 'test_text'}]
        vend_builder = VendBuilder(DATA_BASE, ['test_company_name'])
        additional_requirements = vend_builder.build_additional_requirements()
        self.assertEqual(additional_requirements, [{
            "id": "test_id",
            "type": "vendorAdditionalRequirement",
            "text": "test_text",
            "defaultDocumentIdList": ["4270e05d-8339-4d13-b2ec-2f7a6848a063"],
            "additionalRequirementId": "test_id",
            "vendorId": "test_id",
            "isActive": False
        }])


    @mock.patch('uuid.uuid4', return_value='test_id')
    def test_build_safety_stats(self, mock_uuid4):
        """Test the build_safety_stats method."""
        DATA_BASE.client.results = [{
            "itemType": "vendorSafetyStats",
            "vendorName": "test_company_name"
        }]
        vend_builder = VendBuilder(DATA_BASE, ['test_company_name'])
        safety_stats = vend_builder.build_safety_stats()
        self.assertEqual(safety_stats, {
            "id": "test_id",
            "type": "vendorSafetyStats",
            "vendorId": "test_id"
        })



"""This module is used to test the CommonBuilder class."""

from unittest import TestCase

from eightam_onb_helper.tests.common.cosmos_client_mock import CosmosClientMock
from eightam_db_helper.src.eam_db_helper.db import DatabaseHelper
from eightam_onb_helper.src.eam_onb_helper.vendor_list import CommonBuilder


DATA_BASE = DatabaseHelper(
    {
        'account_uri': 'test_uri',
        'key': 'test_key',
        'db_name': 'test_database',
        'container_name': 'test_container'
    },
    CosmosClientMock
)


COMMON_BUILDER = CommonBuilder(
    DATA_BASE,
    {
        'vendor_id': 'test_vendor_id',
        'vendor_name': 'test_vendor_name',
        'vendor_address': 'test_vendor_address',
        'vendor_phone': 'test_vendor_phone',
        'vendor_icon': 'test_vendor_icon',
        'vendor_admin': 'test_vendor_admin',
        'vendor_overall_rating': 'test_vendor_overall_rating',
        'vendor_subscription_type': 'test_vendor_subscription_type',
        'vendor_dates': 'test_vendor_dates',
        'employer_id': 'test_employer_id',
        'employer_name': 'test_employer_name',
        'employer_address': 'test_employer_address',
        'employer_phone': 'test_employer_phone',
        'employer_icon': 'test_employer_icon',
        'employer_admin': 'test_employer_admin'
    }
)


#pylint: disable=unused-argument
class TestCommonBuilder(TestCase):
    """This class is used to test the CommonBuilder class."""
    def test_build_vendor_list(self):
        """Test the build_vendor_list method."""
        DATA_BASE.client.results = [{'id': '1', 'itemType': 'template'}]
        vendor_list = COMMON_BUILDER.build_vendor_list()
        self.assertEqual(vendor_list, {
            'id': COMMON_BUILDER.item_id,
            'vendorId': 'test_vendor_id',
            'vendorName': 'test_vendor_name',
            'vendorAddress': 'test_vendor_address',
            'vendorPhone': 'test_vendor_phone',
            'vendorIcon': 'test_vendor_icon',
            'vendorAccountAdmin': 'test_vendor_admin',
            'vendorOverallRating': 'test_vendor_overall_rating',
            'vendorSubscriptionType': 'test_vendor_subscription_type',
            'vendorDates': 'test_vendor_dates',
            'employerId': 'test_employer_id',
            'employerName': 'test_employer_name',
            'employerAddress': 'test_employer_address',
            'employerPhone': 'test_employer_phone',
            'employerIcon': 'test_employer_icon',
            'employerAccountAdmin': 'test_employer_admin',
            'type': 'vendorList'
        })



"""This module contains the common builder class for the EAM ONB Helper."""

import uuid
import dataclasses
import time
from eam_db_helper.db import DatabaseHelper


@dataclasses.dataclass
class CommonBuilder:
    """This class is used to build common objects for the EAM ONB Helper."""
    data_base: DatabaseHelper
    vendor_list_data: dict

    # pylint: disable=attribute-defined-outside-init
    def __post_init__(self):
        self.item_id = str(uuid.uuid4())

    def build_vendor_list(self) -> dict:
        """This method is used to build a vendor list."""
        vendor_list = self.data_base.get_result(
            query="SELECT * FROM c WHERE c.type = 'template' AND c.itemType  = 'vendorList'"
        )

        vendor_list['id'] = self.item_id
        vendor_list['vendorId'] = self.vendor_list_data['vendor_id']
        vendor_list['vendorName'] = self.vendor_list_data['vendor_name']
        vendor_list['vendorAddress'] = self.vendor_list_data['vendor_address']
        vendor_list['vendorPhone'] = self.vendor_list_data['vendor_phone']
        vendor_list['vendorIcon'] = self.vendor_list_data['vendor_icon']
        vendor_list['vendorAccountAdmin'] = self.vendor_list_data['vendor_admin']
        vendor_list['vendorOverallRating'] = self.vendor_list_data['vendor_overall_rating']
        vendor_list['vendorSubscriptionType'] = self.vendor_list_data['vendor_subscription_type']
        vendor_list['vendorDates'] = self.vendor_list_data['vendor_dates']

        vendor_list['employerId'] = self.vendor_list_data['employer_id']
        vendor_list['employerName'] = self.vendor_list_data['employer_name']
        vendor_list['employerAddress'] = self.vendor_list_data['employer_address']
        vendor_list['employerPhone'] = self.vendor_list_data['employer_phone']
        vendor_list['employerIcon'] = self.vendor_list_data['employer_icon']
        vendor_list['employerAccountAdmin'] = self.vendor_list_data['employer_admin']

        vendor_list['type'] = 'vendorList'

        del vendor_list['itemType']

        return vendor_list



"""This class is used to test the EmpBuilder class."""

from unittest import mock, TestCase

from eightam_onb_helper.tests.common.cosmos_client_mock import CosmosClientMock
from eightam_onb_helper.src.eam_onb_helper.emp import EmpBuilder
from eightam_db_helper.src.eam_db_helper.db import DatabaseHelper


DATA_BASE = DatabaseHelper(
    {
        'account_uri': 'test_uri',
        'key': 'test_key',
        'db_name': 'test_database',
        'container_name': 'test_container'
    },
    CosmosClientMock
)


#pylint: disable=unused-argument
class TestEmpBuilder(TestCase):
    """This class is used to test the EmpBuilder class."""
    @mock.patch('uuid.uuid4', return_value='test_id')
    def test_format(self, mock_uuid4):
        """This method is used to test the format method."""
        emp_builder = EmpBuilder(DATA_BASE, ['test_company_name'])
        item = {
            'itemType': 'company'
        }
        formatted_item = emp_builder.format(item)
        self.assertEqual(formatted_item, {'id': 'test_id', 'type': 'company'})


    @mock.patch('uuid.uuid4', return_value='test_company_id')
    @mock.patch(
        'eightam_onb_helper.src.eam_onb_helper.common.helpers.generate_sign_up_code',
        return_value='test_sign_up_code'
    )
    @mock.patch('time.time', return_value=1234)
    def test_build_company(self, mock_uuid4, mock_sign_up_cdoe, mock_time):
        """This method is used to test the build_company method."""
        DATA_BASE.client.results = [{'itemType': 'test_item_type', 'dates': {'added': 1234}}]
        emp_builder = EmpBuilder(
            DATA_BASE,
            [
                'test_company_name',
                'test_is_demo',
                'test_website',
                'test_street',
                'test_city',
                'test_province',
                'test_postal_code',
                'test_country',
                'test_office',
                'test_phone',
                '',
                '',
                '',
                'test_email'
            ]
        )

        company = emp_builder.build_company()

        self.assertEqual(company, {
            'id': 'test_company_id',
            'type': 'company',
            'name': 'test_company_name',
            'accountType': 'employer',
            'email': 'test_email',
            'isDemo': 'test_is_demo',
            'signUpCode': 'test_sign_up_code',
            'website': 'test_website',
            'address': {
                'street': 'test_street',
                'city': 'test_city',
                'province': 'test_province',
                'postalCode': 'test_postal_code',
                'country': 'test_country',
                'office': 'test_office'
            },
            'phone': 'test_phone',
            'dates': {'added': 1234}
        })


    @mock.patch('uuid.uuid4', return_value='test_company_id')
    @mock.patch(
        'eightam_onb_helper.src.eam_onb_helper.emp.EmpBuilder.format',
        return_value={'type': 'test_item_type'}
    )
    def test_build_tags(self, mock_uuid, mock_format):
        """This method is used to test the build_tags method."""
        DATA_BASE.client.results = [{'type': 'test_item_type'}]
        emp_builder = EmpBuilder(DATA_BASE, ['test_company_name'])
        tags = emp_builder.build_tags()
        self.assertEqual(tags, {'companyId': 'test_company_id', 'type': 'test_item_type'})


    @mock.patch('uuid.uuid4', return_value='test_company_id')
    @mock.patch(
        'eightam_onb_helper.src.eam_onb_helper.emp.EmpBuilder.format',
        return_value={'type': 'test_item_type'}
    )
    def test_build_employer_doc_config(self, mock_uuid, mock_format):
        """This method is used to test the build_employer_doc_config method."""
        DATA_BASE.client.results = [{'type': 'test_item_type'}]
        emp_builder = EmpBuilder(DATA_BASE, ['test_company_name'])
        employer_doc_config = emp_builder.build_employer_doc_config()
        self.assertEqual(
            employer_doc_config,
            {
                'type': 'test_item_type',
                'employerId': 'test_company_id',
                'employerName': 'test_company_name'
            }
        )



"""This module is used to build the vendor object and associated items for the EAM ONB Helper."""

import uuid
import time

from .common import helpers


class VendBuilder:
    """This class is used to build the Vendor object and associated items for the EAM ONB Helper."""

    def __init__(self, data_base, company_data: list) -> None:
        self.company_id = str(uuid.uuid4())
        self.company_name = company_data[0]
        self.data_base = data_base
        self.company_data = company_data

    def format(self, item) -> dict:
        """This method is used to format the item."""
        item['id'] = str(uuid.uuid4())
        item['type'] = item['itemType']

        if item['type'] != 'company':
            item['vendorId'] = self.company_id
            item['vendorName'] = self.company_name

        del item['itemType']

        return item

    def build_company(self) -> dict:
        """This method is used to build the company."""
        company = self.data_base.get_result(
            query="SELECT * FROM c WHERE c.type = 'template' AND c.itemType = 'company'")

        company['name'] = self.company_name
        company['email'] = self.company_data[14]
        company['isDemo'] = self.company_data[1]
        company['signUpCode'] = helpers.generate_sign_up_code()
        company['subscriptionType'] = self.company_data[2]
        company['website'] = self.company_data[3]
        company['address'] = {
            'street': self.company_data[4],
            'city': self.company_data[5],
            'province': self.company_data[6],
            'postalCode': self.company_data[7],
            'country': self.company_data[8],
            'office': self.company_data[9]
        }
        company['phone'] = self.company_data[10]
        company['dates']['added'] = int(time.time())

        company = self.format(company)

        company['id'] = self.company_id
        return company

    def build_profile_state(self) -> dict:
        """This method is used to build the profile state."""
        return self.format(
            self.data_base.get_result(
                query="SELECT * FROM c WHERE c.type = 'template'"\
                "AND c.itemType = 'vendorPreQualProfileState'"
            )
        )

    def build_payment_state(self) -> dict:
        """This method is used to build the payment state."""
        return self.format(
            self.data_base.get_result(
                query="SELECT * FROM c WHERE c.type = 'template'"\
                "AND c.itemType = 'vendorPreQualPaymentState'"
            )
        )

    def build_documents_state(self) -> dict:
        """This method is used to build the documents state."""
        return self.format(
            self.data_base.get_result(
                query="SELECT * FROM c WHERE c.type = 'template'"\
                "AND c.itemType = 'vendorPreQualDocumentsState'"
            )
        )

    def build_requirements_state(self) -> dict:
        """This method is used to build the requirements state."""
        return self.format(
            self.data_base.get_result(
                query="SELECT * FROM c WHERE c.type = 'template'"\
                "AND c.itemType = 'vendorPreQualRequirementsState'"
            )
        )

    def build_stepper_state(self) -> dict:
        """This method is used to build the stepper state."""
        return self.format(
            self.data_base.get_result(
                query="SELECT * FROM c WHERE c.type = 'template'"\
                "AND c.itemType = 'vendorStepperState'"
            )
        )

    def build_additional_requirements(self) -> list:
        """This method is used to build the additional requirements."""
        additional_requirements = self.data_base.get_results(
            query="SELECT * FROM c WHERE c.type = 'additionalRequirement'")
        for item in additional_requirements:
            item["id"] = str(uuid.uuid4())
            item["type"] = "vendorAdditionalRequirement"
            item["text"] = item['text']
            item["defaultDocumentIdList"] = [
                "4270e05d-8339-4d13-b2ec-2f7a6848a063"]
            item["additionalRequirementId"] = item['id']
            item["vendorId"] = self.company_id
            item["isActive"] = False

        return additional_requirements

    def build_safety_stats(self) -> dict:
        """This method is used to build the safety stats."""
        safety_stats = self.format(self.data_base.get_result(
            query="SELECT * FROM c WHERE c.type = 'template' AND c.itemType = 'vendorSafetyStats'"))
        del safety_stats['vendorName']

        return safety_stats



"""This module contains the CosmosClientMock class."""


# pylint: disable=unused-argument
class CosmosClientMock:
    """This class is used to mock the CosmosClient class."""
    def __init__(self, uri, key):
        self.results = []

    def get_database_client(self, data_base: str):
        """Return the database client."""
        return self

    def get_container_client(self, container: str):
        """Return the container client."""
        return self

    def query_items(
            self, query: str,
            parameters: list = None,
            enable_cross_partition_query: bool = False):
        """Return the query results."""
        results = self.results
        self.results = []

        return results

    def upsert_item(self, body):
        """Add an item to the results."""
        self.results.append(body)
        return body

    def delete_item(self, item_id, partition_key):
        """Delete an item from the results."""
        self.results = None
        


"""This module is used to generate a sign up code."""

import random
import geonamescache


def generate_sign_up_code() -> str:
    """This method is used to generate a sign up code."""
    geo_name = geonamescache.GeonamesCache()
    counties = geo_name.get_us_counties()
    random_county = random.choice(counties)["name"].split(" ")[0]
    random_number = str(random.randint(0, 9999))

    return random_county + random_number
